/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm;

import java.io.InputStream;
import java.util.logging.LogManager;
import java.util.logging.Logger;

import junit.framework.AssertionFailedError;
import junit.framework.TestCase;

public class JbpmTestCase extends TestCase {

  static {
    try {
      InputStream inputStream = JbpmTestCase.class.getClassLoader().getResourceAsStream("logging.properties");
      if (inputStream!=null) {
        LogManager.getLogManager().readConfiguration(inputStream);
      }
    } catch (Exception e) {
      throw new RuntimeException("couldn't initialize logging properly", e);
    }
  }

  public void setUp() {
    log.info("=== starting "+getName()+" =============================");
  }

  public void tearDown() {
    log.info("=== ending "+getName()+" =============================\n");
  }

  public void assertTextPresent(String expected, String value) {
    if ( (value==null)
         || (value.indexOf(expected)==-1)
       ) {
      throw new AssertionFailedError("expected presence of '"+expected+"' but was '"+value+"'");
    }
  }

  protected void runTest() throws Throwable {
    try {
      super.runTest();
    } catch (AssertionFailedError e) {
      log.severe("");
      log.severe("ASSERTION FAILURE: "+e.getMessage());
      log.severe("");
      throw e;
    } catch (Throwable t) {
      t.printStackTrace();
      throw t;
    }
  }

  static Logger log = Logger.getLogger(JbpmTestCase.class.getName());
}
