/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.db;

import java.util.Properties;

import org.jbpm.client.Command;
import org.jbpm.client.CommandExecutor;
import org.jbpm.client.CommandExecutorImpl;
import org.jbpm.env.EnvironmentFactory;


/**
 * @author Tom Baeyens
 */
public abstract class Configuration {

  static Properties databaseConnectionProperties;
  static EnvironmentFactory environmentFactory;
  static CommandExecutor commandExecutor;
  static Command setUpCommand;
  static Command tearDownCommand;
  
  public static void setUp() {
    if (setUpCommand!=null) {
      commandExecutor.execute(setUpCommand);
    }
  }

  public static void tearDown() {
    if (tearDownCommand!=null) {
      commandExecutor.execute(tearDownCommand);
    }
  }

  public static void setEnvironmentFactory(EnvironmentFactory environmentFactory) {
    Configuration.environmentFactory = environmentFactory;
    if (environmentFactory!=null) {
      commandExecutor = new CommandExecutorImpl(environmentFactory);
    } else {
      commandExecutor = null;
    }
  }

  // getters and setters //////////////////////////////////////////////////////
  
  public static CommandExecutor getCommandExecutor() {
    return commandExecutor;
  }
  public static Properties getDatabaseConnectionProperties() {
    return databaseConnectionProperties;
  }
  public static void setDatabaseConnectionProperties(Properties databaseConnectionProperties) {
    Configuration.databaseConnectionProperties = databaseConnectionProperties;
  }
  public static EnvironmentFactory getEnvironmentFactory() {
    return environmentFactory;
  }
  public static Command getSetUpCommand() {
    return setUpCommand;
  }
  public static void setSetUpCommand(Command setUpCommand) {
    Configuration.setUpCommand = setUpCommand;
  }
  public static Command getTearDownCommand() {
    return tearDownCommand;
  }
  public static void setTearDownCommand(Command tearDownCommand) {
    Configuration.tearDownCommand = tearDownCommand;
  }
}
