/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.jbpm.db;

import org.jbpm.client.Command;
import org.jbpm.env.Environment;
import org.jbpm.pvm.Execution;
import org.jbpm.pvm.ExecutionController;
import org.jbpm.pvm.Node;
import org.jbpm.pvm.NodeBehaviour;
import org.jbpm.pvm.Process;
import org.jbpm.pvm.ProcessBuilder;
import org.jbpm.svc.PersistenceService;
import org.jbpm.wire.WireContext;
import org.jbpm.wire.xml.WireParser;

/**
 * @author Charles Souillard
 * @author Guillaume Porcher
 *
 */
public class ProcessDefinitionTest extends CommandTestCase {

  /* USE the command pattern
  public void testProcess() {
    cfg.execute(new Command() {
      public Object execute(Environment environment) throws Exception {

        // Execute logic here in the configured environment
        // One command represents 1 transaction

        return null;
      }
    });
  }
  */

  /* OR USE the try-finally block patter
  public void testProcess() {
    cfg.execute(new Command() {
      public Object execute(Environment environment) throws Exception {

        // Execute logic here in the configured environment
        // One command represents 1 transaction

        return null;
      }
    });
  }
  */
  
  public void testProcess() {
    cfg.execute(new Command() {
      public Object execute(Environment environment) throws Exception {
        Process process = new Process();
        process.setName("persistable");
        
        PersistenceService persistenceSession = (PersistenceService) environment.find(PersistenceService.NAME);
        assertNotNull(persistenceSession);
        return null;
      }
    });
  }
/*
  public void testSimple(){
    PersistenceService persistenceSession = (PersistenceService) environment.find(PersistenceService.NAME);
    assertNotNull(persistenceSession);
    
    Process process = new ProcessBuilder()
      .initial()
        .nodes()
          .node("a")
          .node("b")
          .node("c")
        .endNodes()
    .process();
    
    persistenceSession.persist(process);

    persistenceSession.flush();

    Process p = (Process) persistenceSession.find(Process.class, process.getDbid());
    assertNotNull(p);

    assertSame(process, p);
  }
  
  public void testAccess(){
    PersistenceService persistenceSession = (PersistenceService) environment.find(PersistenceService.NAME);
    assertNotNull(persistenceSession);
    Process process = new ProcessBuilder()
      .initial()
        .nodes()
          .node("a")
          .node("b")
          .node("c")
        .endNodes()
    .process();
    
    persistenceSession.persist(process);

    persistenceSession.flush();

    Environment testEnvironment = environmentFactory.openEnvironment();
    PersistenceService testSession = (PersistenceService) testEnvironment.find(PersistenceService.NAME);
    Process p = (Process) testSession.find(Process.class, process.getDbid());
    Process p2 = (Process) testSession.find(Process.class, process.getDbid() + 1);
    testEnvironment.close();
    
    assertNotNull(p);
    assertNull(p2);
    
    assertNotSame(process, p);
    p.getNodes();
    assertNotNull(p.getInitial());
    assertNotNull(p.getInitial().getNode("a"));
    assertNotNull(p.getInitial().getNode("b"));
    assertNotNull(p.getInitial().getNode("c"));
  }
  
  
  public void testTransition() {
    PersistenceService persistenceSession = (PersistenceService) environment.find(PersistenceService.NAME);
    assertNotNull(persistenceSession);

    Process process = new ProcessBuilder()
      .initial()
        .transition().to("a")
      .node("a")
        .transition().to("b")
      .node("b")
        .transition().to("c")
      .node("c")
      .process();

    persistenceSession.persist(process);

    persistenceSession.flush();

    Environment testEnvironment = environmentFactory.openEnvironment();
    PersistenceService testSession = (PersistenceService) testEnvironment.find(PersistenceService.NAME);
    Process p = (Process) testSession.find(Process.class, process.getDbid());
    testEnvironment.close();
    
    assertNotNull(p);
    assertNotSame(process, p);
    
    Node node = p.getInitial();
    assertEquals(p.getInitial(), node);
    assertNotNull(node.getDefaultTransition());
    assertEquals(node, node.getDefaultTransition().getSource());
    
    node = node.getDefaultTransition().getDestination();
    
    assertEquals(p.getNode("a"), node);
    assertNotNull(node.getDefaultTransition());
    assertEquals(node, node.getDefaultTransition().getSource());
    
    node = node.getDefaultTransition().getDestination();
    
    assertEquals(p.getNode("b"), node);
    assertNotNull(node.getDefaultTransition());
    assertEquals(node, node.getDefaultTransition().getSource());
    
    node = node.getDefaultTransition().getDestination();
    assertEquals(p.getNode("c"), node);
  }
  
  public void testConfiguration() {
    PersistenceService persistenceSession = (PersistenceService) environment.find(PersistenceService.NAME);
    assertNotNull(persistenceSession);

    Process process = new ProcessBuilder()
      .initial("init")
      .process();

    process.setConfiguration(new WireContext(WireParser.parseWireDefinitionFromXmlString(
        "<objects>" +
        "  <string name='bar' value='foo' />" +
        "</objects>"
        )));
    persistenceSession.persist(process);

    process.getInitial().setConfiguration(new WireContext(WireParser.parseWireDefinitionFromXmlString(
        "<objects>" +
    		"  <string name='foo' value='bar' />" +
    		"  <ref name='bar' object='foo' />" +
    		"</objects>"
        )));
    persistenceSession.flush();

    Environment testEnvironment = environmentFactory.openEnvironment();
    PersistenceService testSession = (PersistenceService) testEnvironment.find(PersistenceService.NAME);
    Process p = (Process) testSession.find(Process.class, process.getDbid());
    testEnvironment.close();
    
    assertNotNull(p);
    assertNotSame(process, p);
    
    assertNotNull(p.getConfiguration());
    assertNotNull(p.getConfiguration().has(PersistenceService.NAME));
    
    assertNotNull(p.getInitial());
    assertNotNull(p.getInitial().getConfiguration());
    
    assertEquals("foo", p.getConfiguration().get("bar"));
    assertEquals("bar", p.getInitial().getConfiguration().get("bar"));

  }    
  
  public static class WaitState implements NodeBehaviour {
    private static final long serialVersionUID = 1L;
    
    public WaitState() {
    }

    public void execute(ExecutionController execution) throws Exception {
    }

    public void signal(ExecutionController execution, String signal) throws Exception {
      execution.proceed();
    }

  }
  
  public void testBehaviour() {
    PersistenceService persistenceSession = (PersistenceService) environment.find(PersistenceService.NAME);
    assertNotNull(persistenceSession);

    Process process = new ProcessBuilder()
    .initial().behaviour(new WaitState())
      .transition().to("a")
    .node("a").behaviour(new WaitState())
      .transition().to("b")
    .node("b").behaviour(new WaitState())
      .transition().to("c")
    .node("c").behaviour(new WaitState())
    .process();
  

    persistenceSession.persist(process);

    persistenceSession.flush();

    Environment testEnvironment = environmentFactory.openEnvironment();
    PersistenceService testSession = (PersistenceService) testEnvironment.find(PersistenceService.NAME);
    Process p = (Process) testSession.find(Process.class, process.getDbid());
    testEnvironment.close();
    
    assertNotNull(p);
    assertNotSame(process, p);
        
    Execution execution = p.newExecution();

    assertEquals(p.getInitial(), execution.getNode());
    execution.signal();
    assertEquals(p.getNode("a"), execution.getNode());
    execution.signal();
    assertEquals(p.getNode("b"), execution.getNode());
    execution.signal();
    assertEquals(p.getNode("c"), execution.getNode());

  }
*/
}
